from django import forms

from .models import OTP


class LoginForm(forms.ModelForm):
    code = forms.CharField()

    class Meta:
        model = OTP
        fields = ("phone",)

    def __init__(self, show_code=False, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if show_code or "code" in self.data:
            self.fields["phone"].widget.attrs["readonly"] = True
            self.fields["code"].widget.attrs["autofocus"] = True
        else:
            self.fields["phone"].widget.attrs["autofocus"] = True
            del self.fields["code"]



# Generated by Django 4.1.5 on 2023-01-31 12:08

import allauth_sms.models
from django.db import migrations, models
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="OTP",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "phone",
                    phonenumber_field.modelfields.PhoneNumberField(
                        max_length=128, region=None
                    ),
                ),
                ("ctime", models.DateTimeField(auto_now_add=True)),
                (
                    "code",
                    models.CharField(
                        default=allauth_sms.models.generate_code, max_length=6
                    ),
                ),
                ("is_used", models.BooleanField(default=False)),
            ],
        ),
    ]



from random import randint

from django.db import models
from phonenumber_field.modelfields import PhoneNumberField
from sms import send_sms


def generate_code():
    return randint(100_000, 999_999)


class OTP(models.Model):
    phone = PhoneNumberField()
    ctime = models.DateTimeField(auto_now_add=True)
    code = models.CharField(max_length=6, default=generate_code)
    is_used = models.BooleanField(default=False)

    def __str__(self):
        return str(self.code)

    def send(self):
        send_sms(
            "Your verification code: " + str(self.code),
            None,
            [str(self.phone)],
            fail_silently=False,
        )



from allauth.socialaccount import providers
from allauth.socialaccount.providers.base import Provider, ProviderAccount
from django.shortcuts import reverse


class SMSProvider(Provider):
    id = "sms"
    name = "SMS"

    def get_login_url(self, request, next=None, **kwargs):
        return reverse("account_login_sms")


provider_classes = [SMSProvider]



from datetime import timedelta

from allauth.account.utils import perform_login
from allauth.socialaccount import app_settings
from allauth.socialaccount.models import SocialAccount
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.db import transaction
from django.urls import reverse_lazy
from django.utils.timezone import now
from django.views.generic import FormView

from .forms import LoginForm
from .models import OTP
from .provider import SMSProvider


class LoginView(FormView):
    template_name = "allauth_sms/login.html"
    form_class = LoginForm
    success_url = "/"

    @transaction.atomic
    def form_valid(self, form):
        if "code" in form.cleaned_data:
            # Verify code
            try:
                otp = OTP.objects.get(
                    phone=form.cleaned_data["phone"], code=form.cleaned_data["code"]
                )
            except OTP.DoesNotExist:
                form.add_error("code", "Entered code is invalid.")
                return super().form_invalid(form)
            if otp.is_used:
                form.add_error("code", "Entered code is already used.")
                return super().form_invalid(form)
            if otp.ctime < now() - timedelta(hours=1):
                form.add_error("code", "Entered code is expired.")
                return super().form_invalid(form)
            otp.is_used = True
            otp.save()
            # Find or create user for this phone
            try:
                account = SocialAccount.objects.get(
                    provider=SMSProvider.id, uid=otp.phone
                )
            except SocialAccount.DoesNotExist:
                user = User.objects.create(username=otp.phone)
                account = user.socialaccount_set.create(
                    provider=SMSProvider.id, uid=otp.phone
                )
            return perform_login(
                self.request,
                account.user,
                email_verification=app_settings.EMAIL_VERIFICATION,
            )
        else:
            # Prompt for code
            otp = form.save()
            otp.send()
            form = LoginForm(initial=form.cleaned_data, show_code=True)
            return super().form_invalid(form)
